@@ -11,6 +11,8 @@ use std::str::FromStr;
 use std::time::Duration;
 
 use anyhow::{anyhow, Context};
+use clap::builder::styling::{AnsiColor, Effects};
+use clap::builder::Styles;
 use clap::{self, ArgAction, FromArgMatches, ValueEnum};
 use encoding_rs::Encoding;
 use regex_lite::Regex;
@@ -22,6 +24,15 @@ use crate::redacted::SecretString;
 use crate::request_items::RequestItems;
 use crate::utils::config_dir;
 
+const STYLES: Styles = Styles::styled()
+    .header(AnsiColor::Blue.on_default().effects(Effects::BOLD))
+    .usage(AnsiColor::Blue.on_default().effects(Effects::BOLD))
+    .literal(AnsiColor::Cyan.on_default().effects(Effects::BOLD))
+    .placeholder(AnsiColor::Cyan.on_default())
+    .error(AnsiColor::Red.on_default().effects(Effects::BOLD))
+    .valid(AnsiColor::Green.on_default())
+    .invalid(AnsiColor::Yellow.on_default());
+
 // Some doc comments were copy-pasted from HTTPie
 
 // clap guidelines:
@@ -39,7 +50,8 @@ use crate::utils::config_dir;
     version,
     long_version = long_version(),
     disable_help_flag = true,
-    args_override_self = true
+    args_override_self = true,
+    styles = STYLES,
 )]
 pub struct Cli {
     #[clap(skip)]
@@ -189,7 +201,7 @@ Example: --print=Hb"
     ///  Compression is skipped if it appears that compression ratio is negative.
     ///  Compression can be forced by repeating this option.
     ///
-    ///  Note: Compression cannot be used if the Content-Encoding request header is present.  
+    ///  Note: Compression cannot be used if the Content-Encoding request header is present.
     #[clap(short = 'x', long = "compress", name = "compress", action = ArgAction::Count)]
     pub compress: u8,
 
@@ -392,13 +404,13 @@ Example: --print=Hb"
         long_help = "\
 Generate shell completions or man pages. Possible values are:
 
-    complete-bash
-    complete-elvish
-    complete-fish
-    complete-nushell
-    complete-powershell
-    complete-zsh
-    man
+    complete-bash         Generate completions for bash
+    complete-elvish       Generate completions for elvish
+    complete-fish         Generage completions for fish
+    complete-nushell      Generate completions for nushell
+    complete-powershell   Generate completions for powershell
+    complete-zsh          Generate completions for zsh
+    man                   Generate manual page in roff format
 
 Example: xh --generate=complete-bash > xh.bash",
         conflicts_with = "raw_method_or_url"
@@ -430,8 +442,7 @@ Example: xh --generate=complete-bash > xh.bash",
     ///         Add a query string to the URL.
     ///
     ///     key=value
-    ///         Add a JSON property (--json) or form field (--form) to
-    ///         the request body.
+    ///         Add a JSON property (--json) or form field (--form) to the request body.
     ///
     ///     key:=value
     ///         Add a field with a literal JSON value to the request body.
@@ -441,8 +452,7 @@ Example: xh --generate=complete-bash > xh.bash",
     ///     key@filename
     ///         Upload a file (requires --form or --multipart).
     ///
-    ///         To set the filename and mimetype, ";type=" and
-    ///         ";filename=" can be used respectively.
+    ///         To set the filename and mimetype, ";type=" and ";filename=" can be used respectively.
     ///
     ///         Example: "pfp@ra.jpg;type=image/jpeg;filename=profile.jpg"
     ///
